package model

func RuneTypeToWearPos(typ int64) int64 {
	if typ >= 1 && typ <= 6 {
		return typ - 1
	}
	return -1
}

// 符文表-Rune exported from 符文配置.xlsx
type Rune struct {
	Uid            int64       `json:"Uid"`            // 唯一ID
	Type           int64       `json:"Type"`           // 类型 1-攻击符文 2-均衡符文 3-防御符文 4-战争符文 5-戒律符文 6-潜能符文
	Name           string      `json:"Name"`           // 名称
	FortModeLv     int64       `json:"FortModeLv"`     // 要塞等级要求 0--不限（且不显示）
	Res            string      `json:"Res"`            // 资源
	Grade          int64       `json:"Grade"`          // 品阶
	Quality        int64       `json:"Quality"`        // 品质 1--白 2--绿 3--蓝 4--紫 5--橙 6--红
	EntrySkill1    [][]int64   `json:"EntrySkill1"`    // 第一基础词条 要塞攻击力增加 【【技能类型】，【参数】】
	EntrySkill2    [][]float64 `json:"EntrySkill2"`    // 第二基础词条 【【技能类型】，【参数】】
	Num            int64       `json:"Num"`            // 词条数量
	Disintegration []int64     `json:"Disintegration"` // 分解道具 【道具ID，数量】
}

// 符文升级表-RuneUp exported from 符文配置.xlsx
type RuneLevel struct {
	Uid         int64       `json:"Uid"`         // 唯一ID
	Type        int64       `json:"Type"`        // 部位类型 1-攻击符文 2-均衡符文 3-防御符文 4-战争符文 5-戒律符文 6-潜能符文
	FortModeLv  int64       `json:"FortModeLv"`  // 要塞等级要求  该部位升级要求
	Lv          int64       `json:"Lv"`          // 等级  符文所在坑位等级 初始从【0】级开始
	EntrySkill1 [][]int64   `json:"EntrySkill1"` // 第一基础词条 要塞攻击力增加 【【技能类型】，【参数】】
	EntrySkill2 [][]float64 `json:"EntrySkill2"` // 第二基础词条 【【技能类型】，【参数】】
	Consume     [][]int64   `json:"Consume"`     // 升级消耗 【【道具ID，数量】，【道具ID，数量】】
}

// 符文重铸词条表-RuneEntry exported from 符文配置.xlsx
type RuneEntry struct {
	Uid       int64     `json:"Uid"`       // 唯一ID
	Type      int64     `json:"Type"`      // 部位类型 1-攻击符文 2-均衡符文 3-防御符文 4-战争符文 5-戒律符文 6-潜能符文
	SkillType int64     `json:"SkillType"` // 属性类型
	Down      int64     `json:"Down"`      // 品阶下限
	Up        int64     `json:"Up"`        // 品阶上限
	Range     []float64 `json:"Range"`     // 数值范围  小数值保留4位 【最小值，最大值】
	Weight    int64     `json:"Weight"`    // 权重
}

// 符文重铸消耗表-RuneUse exported from 符文配置.xlsx
type RuneUse struct {
	Uid     int64     `json:"Uid"`     // 唯一ID
	Grade   int64     `json:"Grade"`   // 品阶
	Quality int64     `json:"Quality"` // 品质
	RuneUse [][]int64 `json:"RuneUse"` // 重铸消耗
	LockUse []int64   `json:"LockUse"` // 锁定消耗 (每锁定1条的消耗）
}

// 品阶套装-RuneGrade exported from 符文配置.xlsx
type RuneGrade struct {
	Uid        int64     `json:"Uid"`        // 唯一ID
	Grade      int64     `json:"Grade"`      // 品阶
	Quality    int64     `json:"Quality"`    // 品质
	EntrySkill [][]int64 `json:"EntrySkill"` // 要塞攻击力增加 【【技能类型】，【参数】】
}
